<?php

namespace Bixo\Offplan\Http\Controllers;

use App\Http\Controllers\ResourceController as BaseController;
use Form;
use Bixo\Offplan\Http\Requests\FloorplanRequest;
use Bixo\Offplan\Interfaces\FloorplanRepositoryInterface;
use Bixo\Offplan\Models\Floorplan;

/**
 * Resource controller class for floorplan.
 */
class FloorplanResourceController extends BaseController
{

    /**
     * Initialize floorplan resource controller.
     *
     * @param type FloorplanRepositoryInterface $floorplan
     *
     * @return null
     */
    public function __construct(FloorplanRepositoryInterface $floorplan)
    {
        parent::__construct();
        $this->repository = $floorplan;
        $this->repository
            ->pushCriteria(\Litepie\Repository\Criteria\RequestCriteria::class)
            ->pushCriteria(\Bixo\Offplan\Repositories\Criteria\FloorplanResourceCriteria::class);
    }

    /**
     * Display a list of floorplan.
     *
     * @return Response
     */
    public function index(FloorplanRequest $request)
    {
        $view = $this->response->theme->listView();

        if ($this->response->typeIs('json')) {
            $function = camel_case('get-' . $view);
            return $this->repository
                ->setPresenter(\Bixo\Offplan\Repositories\Presenter\FloorplanPresenter::class)
                ->$function();
        }

        $floorplans = $this->repository->paginate();

        return $this->response->setMetaTitle(trans('offplan::floorplan.names'))
            ->view('offplan::floorplan.index', true)
            ->data(compact('floorplans', 'view'))
            ->output();
    }

    /**
     * Display floorplan.
     *
     * @param Request $request
     * @param Model   $floorplan
     *
     * @return Response
     */
    public function show(FloorplanRequest $request, Floorplan $floorplan)
    {

        if ($floorplan->exists) {
            $view = 'offplan::floorplan.show';
        } else {
            $view = 'offplan::floorplan.new';
        }

        return $this->response->setMetaTitle(trans('app.view') . ' ' . trans('offplan::floorplan.name'))
            ->data(compact('floorplan'))
            ->view($view, true)
            ->output();
    }

    /**
     * Show the form for creating a new floorplan.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function create(FloorplanRequest $request)
    {

        $floorplan = $this->repository->newInstance([]);
        return $this->response->setMetaTitle(trans('app.new') . ' ' . trans('offplan::floorplan.name')) 
            ->view('offplan::floorplan.create', true) 
            ->data(compact('floorplan'))
            ->output();
    }

    /**
     * Create new floorplan.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function store(FloorplanRequest $request)
    {
        try {
            $attributes              = $request->all();
            $attributes['user_id']   = user_id();
            $attributes['user_type'] = user_type();
            $floorplan                 = $this->repository->create($attributes);

            return $this->response->message(trans('messages.success.created', ['Module' => trans('offplan::floorplan.name')]))
                ->code(204)
                ->status('success')
                ->url(guard_url('offplan/floorplan/' . $floorplan->getRouteKey()))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('/offplan/floorplan'))
                ->redirect();
        }

    }

    /**
     * Show floorplan for editing.
     *
     * @param Request $request
     * @param Model   $floorplan
     *
     * @return Response
     */
    public function edit(FloorplanRequest $request, Floorplan $floorplan)
    {
        return $this->response->setMetaTitle(trans('app.edit') . ' ' . trans('offplan::floorplan.name'))
            ->view('offplan::floorplan.edit', true)
            ->data(compact('floorplan'))
            ->output();
    }

    /**
     * Update the floorplan.
     *
     * @param Request $request
     * @param Model   $floorplan
     *
     * @return Response
     */
    public function update(FloorplanRequest $request, Floorplan $floorplan)
    {
        try {
            $attributes = $request->all();

            $floorplan->update($attributes);
            return $this->response->message(trans('messages.success.updated', ['Module' => trans('offplan::floorplan.name')]))
                ->code(204)
                ->status('success')
                ->url(guard_url('offplan/floorplan/' . $floorplan->getRouteKey()))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('offplan/floorplan/' . $floorplan->getRouteKey()))
                ->redirect();
        }

    }

    /**
     * Remove the floorplan.
     *
     * @param Model   $floorplan
     *
     * @return Response
     */
    public function destroy(FloorplanRequest $request, Floorplan $floorplan)
    {
        try {

            $floorplan->delete();
            return $this->response->message(trans('messages.success.deleted', ['Module' => trans('offplan::floorplan.name')]))
                ->code(202)
                ->status('success')
                ->url(guard_url('offplan/floorplan/0'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('offplan/floorplan/' . $floorplan->getRouteKey()))
                ->redirect();
        }

    }

    /**
     * Remove multiple floorplan.
     *
     * @param Model   $floorplan
     *
     * @return Response
     */
    public function delete(FloorplanRequest $request, $type)
    {
        try {
            $ids = hashids_decode($request->input('ids'));

            if ($type == 'purge') {
                $this->repository->purge($ids);
            } else {
                $this->repository->delete($ids);
            }

            return $this->response->message(trans('messages.success.deleted', ['Module' => trans('offplan::floorplan.name')]))
                ->status("success")
                ->code(202)
                ->url(guard_url('offplan/floorplan'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->status("error")
                ->code(400)
                ->url(guard_url('/offplan/floorplan'))
                ->redirect();
        }

    }

    /**
     * Restore deleted floorplans.
     *
     * @param Model   $floorplan
     *
     * @return Response
     */
    public function restore(FloorplanRequest $request)
    {
        try {
            $ids = hashids_decode($request->input('ids'));
            $this->repository->restore($ids);

            return $this->response->message(trans('messages.success.restore', ['Module' => trans('offplan::floorplan.name')]))
                ->status("success")
                ->code(202)
                ->url(guard_url('/offplan/floorplan'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->status("error")
                ->code(400)
                ->url(guard_url('/offplan/floorplan/'))
                ->redirect();
        }

    }

}
